package com.googlecode.smartgwtmavenplugin;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

/**
 * Builds a JAR file from the contents of a directory on the filesystem (recursively).
 * Adapted from http://stackoverflow.com/questions/1281229/how-to-use-jaroutputstream-to-create-a-jar-file (question answered at 
 * http://stackoverflow.com/a/1281295)
 */
public class JarBuilder {
	
	public static void build(File directory, File output) throws IOException {
		Manifest manifest = new Manifest();
		manifest.getMainAttributes().put(Attributes.Name.MANIFEST_VERSION, "1.0");
		JarOutputStream target = new JarOutputStream(new FileOutputStream(output), manifest);
		add(directory, directory, target);
		target.close();
	}

	private static void add(File root, File source, JarOutputStream target) throws IOException {

		String relativePath = root.toURI().relativize(source.toURI()).getPath().replace("\\", "/");
		
		BufferedInputStream in = null;
		try {
			if (source.isDirectory()) {

				if (!relativePath.endsWith("/")) {
					relativePath += "/";
				}
				JarEntry entry = new JarEntry(relativePath);
				entry.setTime(source.lastModified());
				target.putNextEntry(entry);
				target.closeEntry();

				for (File nestedFile : source.listFiles()) {
					add(root, nestedFile, target);
				}
				return;
			}

			JarEntry entry = new JarEntry(relativePath);
			entry.setTime(source.lastModified());
			target.putNextEntry(entry);
			in = new BufferedInputStream(FileUtils.openInputStream(source));
			IOUtils.copy(in, target);
			target.closeEntry();

		} finally {
			IOUtils.closeQuietly(in);
		}
	}
}
